# graphe_value.spec
# Specification algebrique des graphes dont les arcs sont values par des entiers
# Auteur : VOURIOT Thierry
# Date : 15 janvier 2002

# Sorte S definissant l'egalite
spec EG etend BASE

	sorte S

	operations

		==: S S -> Bool		#Egalite
		!=: S S -> Bool		#Difference

	axiomes

		(x==x) = vrai
		(x!=y) = non (x==y)

fspec

# Definition de la sorte avec les operations "standards"
spec GRAPHE(EG)

	sorte Graphe

	operations

		grnouv: -> Graphe		#Creation du graphe vide
		adjs: Graphe S -> Graphe	#Adjonction d'un sommet
		adja: Graphe S S Ent -> Graphe	#Adjonction d'un arc
		supps: Graphe S -> Graphe	#Suppression d'un sommet
		suppa: Graphe S S -> Graphe	#Suppression d'un arc
		di: Graphe S -> Nat		#Degre interieur
		de: Graphe S -> Nat		#Degre exterieur
		exs: Graphe S -> Bool		#Existence d'un sommet
		exa: Graphe S S -> Bool		#Existence d'un arc
		va: Graphe S S -> Ent		#Valuation d'un arc
		modva: Graphe S S Ent -> Graphe	#Modification de la valuation d'un arc


	preconditions 

		pre adjs(g,x) = non exs(g,x)
		pre adja(g,x,y,v) = exs(g,x) et exs(g,y) et non exa(g,x,y)
		pre supps(g,x) = exs(g,x) et di(g,x)==0 et de(g,x)==0
		pre suppa(g,x,y) = exs(g,x) et exs(g,y) et exa(g,x,y)
		pre di(g,x) = pre de(g,x) = exs(g,x)
		pre exa(g,x,y) = exs(g,x) et exs(g,y)
		pre va(g,x,y) = exs(g,x) et exs(g,y)
		pre modva(g,x,y) = exs(g,x) et exs(g,y)

		
	axiomes 
		#Constructeurs grnouv adjs et adja

		exs(grnouv,z) = faux
		exs(adjs(g,x),z) = x==z ou exs(g,z)
		exs(adja(g,x,y),z) = exs(g,z)

		exa(grnouv,z,t) = faux
		exa(adjs(g,x),z,t) = exa(g,z,t)
		exa(adja(g,x,y),z,t) = (x,y)==(z,t) ou exa(g,z,t)

		di(adjs(g,x),z) = si x==z alors 0
				sinon di(g,z) fsi
		di(adja(g,x,y),z) = si y==z alors di(g,z)+1
				sinon di(g,z) fsi

		de(adjs(g,x),z) = si x==z alors 0
				sinon de(g,z) fsi
		de(adja(g,x,y),z) = si x==z alors de(g,z)+1
				sinon de(g,z) fsi

		supps(adjs(g,x),z) = si x==z alors g
				sinon adjs(supps(g,z),x) fsi
		supps(adja(g,x,y),z) = adja(supps(g,z),x,y)

		suppa(adjs(g,x),z,t) = adjs(suppa(g,z,t),x)
		suppa(adja(g,x,y),z,t) = si (x,y)==(z,t) alors g
				sinon adja(suppa(g,z,t),x,y) fsi

		va(adjs(g,x),z,t) = va(g,z,t)
		va(adja(g,x,y,v),z,t) = si (x,y)==(z,t) alors v
					sinon va(g,z,t) fsi
		
		modva(adjs(g,x),z,t,val) = modva(g,z,t)
		modva(adja(g,x,y,v),z,t,val) = si (x,y)==(z,t) alors adja(g,x,y,val)
					sinon modva(g,z,t,val) fsi
		
fspec

# Ajout d'operations sur les graphes
spec GRAPHE1(EG) etend GRAPHE(EG),ENSEMBLE(EG,BSUP)

	operations

		es: Graphe -> Ens		#Ensemble des sommets
		esucc: Graphe S -> Ens		#Ensemble des successeurs d'un sommet
		epred: Graphe S -> Ens		#Ensemble des predecesseurs d'un sommet


	preconditions 

		pre esucc(g,x) = pre epred(g,x) = exs(g,x)
		

	axiomes 
		#Constructeurs grnouv adjs et adja

		es(grnouv) = ensnouv
		es(adjs(g,x)) = ins(es(g),x)
		es(adja(g,x,y)) = es(g)

		esucc(adjs(g,x),z) = si x==z alors ensnouv
				sinon esucc(g,z) fsi
		esucc(adja(g,x,y),z) = si z==x alors ins(esucc(g,z),y)
					sinon esucc(g,z) fsi

		epred(adjs(g,x),z) = si x==z alors ensnouv
				sinon epred(g,z) fsi
		epred(adja(g,x,y),z) = si z==y alors ins(epred(g,z),x)
					sinon epred(g,z) fsi
	
fspec

