/* graphev_ma.c
 * Fichier source pour l'implantation des graphes dont les
 * arcs sont values par des entiers (matrice d'adjacences)
 * Auteur : VOURIOT Thierry
 * Date : 16 janvier 2002
 */

#include "graphev_ma.h"

/* Creation du graphe vide */
Graphev grnouv()
{
	int i,j;
	Graphev g = MALLOC(struct sGraphe);
	g->sommets=ensnouv();
	g->arcs=MALLOCN(int,N*N);
	for(i=0; i<N;i++)
		for(j=0; j<N ; j++)
			g->arcs[i*N+j]=-1;
	return g;
}
	
/* Adjonction d'un sommet
 * pre adjs(g,x) = non exs(g,x) et 0<=x<=N-1
 */ 
Graphev adjs(Graphev g, Nat s)
{
	g->sommets = ins(g->sommets,s);
	return g;
}

/* Adjonction d'un arc
 * pre adja(g,x,y,v) = exs(g,x) et exs(g,y) et non exa(g,x,y)
 */	
Graphev adja(Graphev g, Nat x,Nat y, int val)
{
	g->arcs[x*N+y] = val;
	return g;
}

/* Suppression d'un sommet
 * pre supps(g,x) = exs(g,x) et di(g,x)==0 et de(g,x)==0
 */
Graphev supps(Graphev g, Nat x)
{
	g->sommets=supp(g->sommets,x);
	return g;
}

/* Suppression d'un arc
 * pre suppa(g,x,y) = exs(g,x) et exs(g,y) et exa(g,x,y)
 */
Graphev suppa(Graphev g, Nat x, Nat y)
{
	g->arcs[x*N+y]=-1;
	return g;
}

/* Degre interieur
 * pre di(g,x) = pre de(g,x) = exs(g,x)
 */
Nat di(Graphev g, Nat x)
{
	int i;
	Nat d=0;
	for(i=0;i<N;i++)
		if (g->arcs[x+i*N]) d++;
	return d;
}

/* Degre exterieur
 * pre di(g,x) = pre de(g,x) = exs(g,x)
 */
Nat de(Graphev g, Nat x)
{
	int i;
	Nat d=0;
	for(i=0;i<N;i++)
		if (g->arcs[i+x*N]) d++;
	return d;
}

/* Existance d'un sommet 
 * pre exs(g,x) = 0<=x<=N-1
 */
Bool exs(Graphev g, Nat x)
{
	return app(g->sommets,x);
}
	
/* Existence d'un arc
 * pre exa(g,x,y) = exs(g,x) et exs(g,y)
 */
Bool exa(Graphev g, Nat x, Nat y)
{
	return (g->arcs[x*N+y] != -1);
}

/* Valuation d'un arc
 * pre va(g,x,y) = exs(g,x) et exs(g,y)
 */
int va(Graphev g, Nat x, Nat y)
{
	return g->arcs[x*N+y];
}

/* Modification de la valuation d'un arc
 * pre modva(g,x,y) = exs(g,x) et exs(g,y)
 */
Graphev modva(Graphev g, Nat x, Nat y,int val)
{
	g->arcs[x*N+y]=val;
	return g;
}

/* Ensemble des successeurs d'un sommet
 * pre esucc(g,x) = pre epred(g,x) = exs(g,x)
 */
Ens esucc(Graphev g, Nat x)
{	Ens e=g->sommets;
	Ens f=ensnouv();
	int j=0;
	while(j<N)
	{	
		if (app(e,j) && exa(g,x,j))
			f = ins(f,j);
		j++;
	}
	return f;
}

/* Ensemble des predecesseurs d'un sommet
 * pre esucc(g,x) = pre epred(g,x) = exs(g,x)
 */	
Ens epred(Graphev g, Nat x)
{	Ens e=g->sommets;
	Ens f=ensnouv();
	int j=0;
	while(j<N)
	{	
		if (app(e,j) && exa(g,j,x))
			f=ins(f,j);
		j++;}
	return f;
}

/* Affiche la matrice d'adjacences */
void affMat(Graphev g)
{
	int i,j;
	printf("  ");
	aff(g->sommets);
	for (i=0;i<N;i++)
		if (exs(g,i))
		{
			printf("%2d| ",i);
			for (j=0;j<N;j++)
			{
				if (exs(g,j))
					printf("%2d ",va(g,i,j));
			}
			printf("\n");
		}
}

int main()
{
	Graphev g = grnouv();
	g = adjs(adjs(adjs(adjs(adjs(adjs(adjs(g,2),4),6),8),10),12),0);
	g = adja(adja(adja(adja(adja(g,10,2,3),0,2,2),12,6,10),10,12,8),4,6,7);
	g = adja(adja(adja(adja(adja(g,10,3,2),2,0,2),6,12,10),12,10,8),6,4,7);
	printf("\nExemple de graphe value (matrice d'adjacences) :\n");affMat(g);
}
