/* graphev_ma.h
 * Fichier d'en tete pour l'implantation des graphes dont les
 * arcs sont values par des entiers de 0  N-1 (matrice d'adjacences)
 * Auteur : VOURIOT Thierry
 * Date : 16 janvier 2002
 */

#include "base.h"
#include "ens_tab.h"

/* Implantation sous la forme d'un ensemble de sommets
 * et d'une matrice d'adjacence pour les arcs 
 */
typedef struct sGraphe
{
	Ens sommets;
	int *arcs;
} *Graphev;

/* Creation du graphe vide */	
extern Graphev grnouv(void);

/* Adjonction d'un sommet
 * pre adjs(g,x) = non exs(g,x) et 0<=x<=N-1
 */
extern Graphev adjs(Graphev, Nat);

/* Adjonction d'un arc
 * pre adja(g,x,y,v) = exs(g,x) et exs(g,y) et non exa(g,x,y)
 */
extern Graphev adja(Graphev, Nat, Nat, int);

/* Suppression d'un sommet
 * pre supps(g,x) = exs(g,x) et di(g,x)==0 et de(g,x)==0
 */
extern Graphev supps(Graphev, Nat);

/* Suppression d'un arc
 * pre suppa(g,x,y) = exs(g,x) et exs(g,y) et exa(g,x,y)
 */
extern Graphev suppa(Graphev, Nat, Nat);

/* Degre interieur
 * pre di(g,x) = pre de(g,x) = exs(g,x)
 */
extern Nat di(Graphev, Nat);

/* Degre exterieur
 * pre di(g,x) = pre de(g,x) = exs(g,x)
 */
extern Nat de(Graphev, Nat);

/* Existance d'un sommet 
 * pre exs(g,x) = 0<=x<=N-1
 */
extern Bool exs(Graphev, Nat);

/* Existence d'un arc
 * pre exa(g,x,y) = exs(g,x) et exs(g,y)
 */
extern Bool exa(Graphev, Nat, Nat);

/* Valuation d'un arc
 * pre va(g,x,y) = exs(g,x) et exs(g,y)
 */
extern int va(Graphev, Nat, Nat);

/* Modification de la valuation d'un arc
 * pre modva(g,x,y) = exs(g,x) et exs(g,y)
 */
extern Graphev modva(Graphev, Nat, Nat, int);

/* Ensemble des successeurs d'un sommet
 * pre esucc(g,x) = pre epred(g,x) = exs(g,x)
 */
extern Ens esucc(Graphev, Nat);

/* Ensemble des predecesseurs d'un sommet
 * pre esucc(g,x) = pre epred(g,x) = exs(g,x)
 */
extern Ens epred(Graphev, Nat);

/* Affiche la matrice d'adjacences */
extern void affMat(Graphev);
